import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/core';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { BaseController } from '../../frame/BaseController';
import { omit, pick } from 'lodash';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { Context } from '@midwayjs/koa';
import { IncubatorService } from '../../service/users/incubator.service';
import { QVO_incubator, FVO_incubator, CVO_incubator, UVAO_incubator } from '../../interface/users/incubator.interface';
import { ServiceUtilService } from '../../service/serviceUtil.service';
const namePrefix = '孵化器';
@ApiTags(['孵化器'])
@Controller('/incubator', { tagName: namePrefix })
export class IncubatorController implements BaseController {
  @Inject()
  service: IncubatorService;
  @Inject()
  ctx: Context;

  @Inject()
  serviceUtil: ServiceUtilService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_incubator })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_incubator })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_incubator(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_incubator })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    this.serviceUtil.fillIdentity(data, 'Incubator');
    const result = new CVO_incubator(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_incubator })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    this.serviceUtil.fillIdentity(data, 'Incubator');
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
